@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: dochandler.cpp,v 1.33 2002/10/13 21:01:58 dimitri Exp $
+ * $Id$
  *
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -551,16 +551,13 @@ void TocItemHandler::endTocItem()
 //----------------------------------------------------------------------
 
 ParameterHandler::ParameterHandler(IBaseHandler *parent) : 
-  m_parent(parent), m_description(0)
+  m_parent(parent)
 {
   addEndHandler("parametername",this,&ParameterHandler::endParameterName);
-  addEndHandler("parameterdescription",this,&ParameterHandler::endParameterDescription);
-  addStartHandler("para",this,&ParameterHandler::startParagraph);
 }
 
 ParameterHandler::~ParameterHandler()
 {
-  delete m_description;
 }
 
 void ParameterHandler::startParameterName(const QXmlAttributes& /*attrib*/)
@@ -576,23 +573,57 @@ void ParameterHandler::endParameterName()
   m_parent->setDelegate(0);
 }
 
-void ParameterHandler::startParameterDescription(const QXmlAttributes& /*attrib*/)
+//----------------------------------------------------------------------
+// ParameterListHandler
+//----------------------------------------------------------------------
+
+ParameterItemHandler::ParameterItemHandler(IBaseHandler *parent) 
+  : m_parent(parent)
+{
+  addEndHandler("parameteritem",this,&ParameterItemHandler::endParameterItem);
+  addStartHandler("parameternamelist");
+  addEndHandler("parameternamelist");
+  addStartHandler("parametername",this,&ParameterItemHandler::startParameterName);
+  addStartHandler("parameterdescription");
+  addEndHandler("parameterdescription");
+  addStartHandler("para",this,&ParameterItemHandler::startParagraph);
+  m_parameters.setAutoDelete(TRUE);
+  m_description = 0;
+}
+
+ParameterItemHandler::~ParameterItemHandler()
+{
+  delete m_description;
+}
+
+void ParameterItemHandler::startParameterItem(const QXmlAttributes&)
 {
   m_parent->setDelegate(this);
 }
 
-void ParameterHandler::endParameterDescription()
+void ParameterItemHandler::endParameterItem()
 {
   m_parent->setDelegate(0);
 }
 
-void ParameterHandler::startParagraph(const QXmlAttributes& attrib)
+void ParameterItemHandler::startParameterName(const QXmlAttributes& attrib)
+{
+  ParameterHandler *param = new ParameterHandler(this);
+  m_parameters.append(param);
+  param->startParameterName(attrib);
+}
+
+void ParameterItemHandler::startParagraph(const QXmlAttributes& attrib)
 {
-  ASSERT(m_description==0);
   m_description = new ParagraphHandler(this);
   m_description->startParagraph(attrib);
 }
 
+IDocIterator *ParameterItemHandler::paramNames() const
+{
+  return new ParameterItemIterator(*this);
+}
+
 //----------------------------------------------------------------------
 // ParameterListHandler
 //----------------------------------------------------------------------
@@ -601,12 +632,8 @@ ParameterListHandler::ParameterListHandler(IBaseHandler *parent)
   : m_parent(parent)
 {
   addEndHandler("parameterlist",this,&ParameterListHandler::endParameterList);
-  addStartHandler("parametername",this,&ParameterListHandler::startParameterName);
-  addStartHandler("parameterdescription",this,&ParameterListHandler::startParameterDescription);
-  addStartHandler("title");
-  addEndHandler("title");
-  m_parameters.setAutoDelete(TRUE);
-  m_curParam=0;
+  addStartHandler("parameteritem",this,&ParameterListHandler::startParameterItem);
+  m_paramItems.setAutoDelete(TRUE);
 }
 
 ParameterListHandler::~ParameterListHandler()
@@ -632,17 +659,11 @@ void ParameterListHandler::endParameterList()
   m_parent->setDelegate(0);
 }
 
-void ParameterListHandler::startParameterName(const QXmlAttributes& attrib)
+void ParameterListHandler::startParameterItem(const QXmlAttributes& attrib)
 {
-  m_curParam = new ParameterHandler(this);
-  m_parameters.append(m_curParam);
-  m_curParam->startParameterName(attrib);
-}
-
-void ParameterListHandler::startParameterDescription(const QXmlAttributes& attrib)
-{
-  ASSERT(m_curParam!=0);
-  m_curParam->startParameterDescription(attrib);
+  ParameterItemHandler *paramItem = new ParameterItemHandler(this);
+  m_paramItems.append(paramItem);
+  paramItem->startParameterItem(attrib);
 }
 
 IDocIterator *ParameterListHandler::params() const
@@ -1097,7 +1118,6 @@ void HighlightHandler::startSpace(const QXmlAttributes&)
 
 void HighlightHandler::addTextNode()
 {
-  printf("m_curString=`%s'\n",m_curString.data());
   if (!m_curString.isEmpty())
   {
     m_children.append(new TextNode(m_curString,IDocMarkup::Normal,0));
@@ -1658,28 +1678,37 @@ ParagraphHandler::ParagraphHandler(IBaseHandler *parent)
   m_children.setAutoDelete(TRUE);
 
   m_markupHandler = new MarkupHandler(m_children,m_curString);
+
+  // preformatted
   setFallBackHandler(m_markupHandler);
 
   addEndHandler("para",this,&ParagraphHandler::endParagraph);
 
-  addStartHandler("itemizedlist",this,&ParagraphHandler::startItemizedList);
-  addStartHandler("orderedlist",this,&ParagraphHandler::startOrderedList);
-  addStartHandler("parameterlist",this,&ParagraphHandler::startParameterList);
-  addStartHandler("simplesect",this,&ParagraphHandler::startSimpleSect);
-  addStartHandler("ref",this,&ParagraphHandler::startRef);
-  addStartHandler("variablelist",this,&ParagraphHandler::startVariableList);
-  addStartHandler("hruler",this,&ParagraphHandler::startHRuler);
   addStartHandler("linebreak",this,&ParagraphHandler::startLineBreak);
+  addStartHandler("hruler",this,&ParagraphHandler::startHRuler);
+  addStartHandler("programlisting",this,&ParagraphHandler::startProgramListing);
+  addStartHandler("verbatim",this,&ParagraphHandler::startVerbatim);
+  addStartHandler("indexentry",this,&ParagraphHandler::startIndexEntry);
+  addStartHandler("orderedlist",this,&ParagraphHandler::startOrderedList);
+  addStartHandler("itemizedlist",this,&ParagraphHandler::startItemizedList);
+  addStartHandler("simplesect",this,&ParagraphHandler::startSimpleSect);
+  // TODO: title
+  addStartHandler("variablelist",this,&ParagraphHandler::startVariableList);
+  addStartHandler("table",this,&ParagraphHandler::startTable);
+  // TODO: heading
+  addStartHandler("image",this,&ParagraphHandler::startImage);
+  addStartHandler("dotfile",this,&ParagraphHandler::startDotFile);
+  addStartHandler("toclist",this,&ParagraphHandler::startTocList);
+  // TODO: language???
+  addStartHandler("parameterlist",this,&ParagraphHandler::startParameterList);
+  // TODO: xrefsect
+  addStartHandler("copydoc",this,&ParagraphHandler::startCopyDoc);
+
+  addStartHandler("ref",this,&ParagraphHandler::startRef);
   addStartHandler("ulink",this,&ParagraphHandler::startULink);
   addStartHandler("email",this,&ParagraphHandler::startEMail);
   addStartHandler("link",this,&ParagraphHandler::startLink);
-  addStartHandler("programlisting",this,&ParagraphHandler::startProgramListing);
   addStartHandler("formula",this,&ParagraphHandler::startFormula);
-  addStartHandler("image",this,&ParagraphHandler::startImage);
-  addStartHandler("dotfile",this,&ParagraphHandler::startDotFile);
-  addStartHandler("indexentry",this,&ParagraphHandler::startIndexEntry);
-  addStartHandler("table",this,&ParagraphHandler::startTable);
-  addStartHandler("verbatim",this,&ParagraphHandler::startVerbatim);
   addStartHandler("latexonly",this,&ParagraphHandler::startHtmlOnly);
   addStartHandler("htmlonly",this,&ParagraphHandler::startLatexOnly);
   addStartHandler("umlaut",this,&ParagraphHandler::startUmlaut);
@@ -1693,8 +1722,6 @@ ParagraphHandler::ParagraphHandler(IBaseHandler *parent)
   addStartHandler("nbsp",this,&ParagraphHandler::startNbsp);
   addStartHandler("copy",this,&ParagraphHandler::startCopy);
   addStartHandler("anchor",this,&ParagraphHandler::startAnchor);
-  addStartHandler("copydoc",this,&ParagraphHandler::startCopyDoc);
-  addStartHandler("toclist",this,&ParagraphHandler::startTocList);
 }
 
 ParagraphHandler::~ParagraphHandler()
@@ -1734,9 +1761,9 @@ void ParagraphHandler::startOrderedList(const QXmlAttributes& attrib)
 void ParagraphHandler::startParameterList(const QXmlAttributes& attrib)
 {
   addTextNode();
-  ParameterListHandler *parListHandler = new ParameterListHandler(this);
-  parListHandler->startParameterList(attrib);
-  m_children.append(parListHandler);
+  ParameterListHandler *listHandler = new ParameterListHandler(this);
+  listHandler->startParameterList(attrib);
+  m_children.append(listHandler);
 }
 
 void ParagraphHandler::startSimpleSect(const QXmlAttributes& attrib)
@@ -2016,11 +2043,11 @@ DocSectionHandler::DocSectionHandler(IBaseHandler *parent,int level)
   if (level<6) 
   {
     sectionKey.sprintf("sect%d",level+1);
-    addStartHandler(sectionKey,this,&DocSectionHandler::startSubSection);
+    addStartHandler(sectionKey.utf8(),this,&DocSectionHandler::startSubSection);
   }
   addStartHandler("internal",this,&DocSectionHandler::startInternal);
   sectionKey.sprintf("sect%d",level);
-  addEndHandler(sectionKey,this,&DocSectionHandler::endDocSection);
+  addEndHandler(sectionKey.utf8(),this,&DocSectionHandler::endDocSection);
 }
 
 DocSectionHandler::~DocSectionHandler()
@@ -2093,7 +2120,7 @@ DocInternalHandler::DocInternalHandler(IBaseHandler *parent,int level)
   addStartHandler("para",this,&DocInternalHandler::startParagraph);
   QString sectionKey;
   sectionKey.sprintf("sect%d",level+1);
-  addStartHandler(sectionKey,this,&DocInternalHandler::startSubSection);
+  addStartHandler(sectionKey.utf8(),this,&DocInternalHandler::startSubSection);
   addEndHandler("internal",this,&DocInternalHandler::endInternal);
 }
 
@@ -2148,12 +2175,14 @@ DocHandler::DocHandler(IBaseHandler *parent) : m_parent(parent)
 
   addEndHandler("briefdescription",this,&DocHandler::endDoc);
   addEndHandler("detaileddescription",this,&DocHandler::endDoc);
-  addEndHandler("internal");
+  addEndHandler("inbodydescription",this,&DocHandler::endDoc);
+  //addEndHandler("internal"); // TODO: implement this as a section
+  addStartHandler("internal",this,&DocHandler::startInternal);
 
   addStartHandler("para",this,&DocHandler::startParagraph);
   addStartHandler("sect1",this,&DocHandler::startSect1);
   addStartHandler("title",this,&DocHandler::startTitle);
-  addStartHandler("internal");
+  //addStartHandler("internal");
 }
 
 DocHandler::~DocHandler()
@@ -2193,8 +2222,19 @@ void DocHandler::startTitle(const QXmlAttributes& attrib)
   m_children.append(titleHandler);
 }
 
+void DocHandler::startInternal(const QXmlAttributes& attrib)
+{
+  m_internal = new DocInternalHandler(this,1);
+  m_internal->startInternal(attrib);
+}
+
 IDocIterator *DocHandler::contents() const
 {
   return new DocIterator(*this);
 }
 
+IDocInternal *DocHandler::internal() const 
+{ 
+  return m_internal; 
+}
+
